@@ -1324,6 +1324,13 @@ nsHTMLInputElement::HandleDOMEvent(nsIPresContext* aPresContext,
     }
   }
 
+ // Don't allow mutation events which are targeted somewhere inside
+ // <input>, except if they are dispatched to the element itself.
+ if (!(NS_EVENT_FLAG_INIT & aFlags) &&
+     aEvent->eventStructType == NS_MUTATION_EVENT) {
+   return NS_OK;
+ }
+
   //
   // Web pages expect the value of a radio button or checkbox to be set
   // *before* onclick fires, and they expect that if they set the value
@@ -1719,8 +1726,14 @@ nsHTMLInputElement::ParseAttribute(nsIAtom* aAttribute,
       return PR_FALSE;
     }
 
-    mType = aResult.GetEnumValue();
-    if (mType == NS_FORM_INPUT_FILE) {
+    // Make sure to do the check for newType being NS_FORM_INPUT_FILE and the
+    // corresponding SetValueInternal() call _before_ we set mType.  That way
+    // the logic in SetValueInternal() will work right (that logic makes
+    // assumptions about our frame based on mType, but we won't have had time
+    // to recreate frames yet -- that happens later in the SetAttr()
+    // process).
+    PRInt8 newType = aResult.GetEnumValue();
+    if (newType == NS_FORM_INPUT_FILE) {      
       // If the type is being changed to file, set the element value
       // to the empty string. This is for security.
       // Call SetValueInternal so that this doesn't accidentally get caught
@@ -1728,6 +1741,8 @@ nsHTMLInputElement::ParseAttribute(nsIAtom* aAttribute,
       SetValueInternal(EmptyString(), nsnull);
     }
 
+    mType = newType;
+    
     return PR_TRUE;
   }
   if (aAttribute == nsHTMLAtoms::width) {
